@@ -151,12 +151,22 @@ pub fn statemachine(input: TokenStream) -> TokenStream {
             let arg = state_data_types.get(next_state_name);
             let enter_fn_name = format_ident!("{}_{}", "on_enter", next_state_name.to_string().to_case(Case::Snake));
 
-            let exit_call = match state_data_types.get(state_name) {
-                Some(_) => quote! {
-                    self.observer.#exit_fn_name(&self.id, State::#next_state_name, self.state.data()).await.map_err(|e| TransitionError::ObserverError(e))?;
+            let exit_call = match shared_data_type {
+                Some(_) => match state_data_types.get(state_name) {
+                    Some(_) => quote! {
+                        self.observer.#exit_fn_name(&self.id, State::#next_state_name, &self.data, &self.state.data).await.map_err(|e| TransitionError::ObserverError(e))?;
+                    },
+                    None => quote! {
+                        self.observer.#exit_fn_name(&self.id, State::#next_state_name, &self.data).await.map_err(|e| TransitionError::ObserverError(e))?;
+                    }
                 },
-                None => quote! {
-                    self.observer.#exit_fn_name(&self.id, State::#next_state_name).await.map_err(|e| TransitionError::ObserverError(e))?;
+                None => match state_data_types.get(state_name) {
+                    Some(_) => quote! {
+                        self.observer.#exit_fn_name(&self.id, State::#next_state_name, &self.state.data).await.map_err(|e| TransitionError::ObserverError(e))?;
+                    },
+                    None => quote! {
+                        self.observer.#exit_fn_name(&self.id, State::#next_state_name).await.map_err(|e| TransitionError::ObserverError(e))?;
+                    }
                 }
             };
 
@@ -165,9 +175,9 @@ pub fn statemachine(input: TokenStream) -> TokenStream {
                     Some(_) => quote! {
                         impl<T: Observer + Send> #parent_name<#state_name, T> {
                             pub async fn #event(mut self, data: #a) -> Result<#parent_name<#next_state_name, T>, TransitionError<T::Error>> {
-                                self.observer.on_transition(&self.id, State::#state_name, State::#next_state_name, Some(&data)).await.map_err(|e| TransitionError::ObserverError(e))?;
+                                self.observer.on_transition(&self.id, State::#state_name, State::#next_state_name, Some(&self.data), Some(&data)).await.map_err(|e| TransitionError::ObserverError(e))?;
                                 #exit_call
-                                self.observer.#enter_fn_name(&self.id, Some(State::#state_name), &data).await.map_err(|e| TransitionError::ObserverError(e))?;
+                                self.observer.#enter_fn_name(&self.id, Some(State::#state_name), &self.data, &data).await.map_err(|e| TransitionError::ObserverError(e))?;
                                 Ok(#parent_name::<#next_state_name, T>::new(self.id, #next_state_name::new(data), self.data, self.observer))
                             }
                         }
@@ -175,7 +185,7 @@ pub fn statemachine(input: TokenStream) -> TokenStream {
                     None => quote! {
                         impl<T: Observer + Send> #parent_name<#state_name, T> {
                             pub async fn #event(mut self, data: #a) -> Result<#parent_name<#next_state_name, T>, TransitionError<T::Error>> {
-                                self.observer.on_transition(&self.id, State::#state_name, State::#next_state_name, Some(&data)).await.map_err(|e| TransitionError::ObserverError(e))?;
+                                self.observer.on_transition(&self.id, State::#state_name, State::#next_state_name, Option::<()>::None, Some(&data)).await.map_err(|e| TransitionError::ObserverError(e))?;
                                 #exit_call
                                 self.observer.#enter_fn_name(&self.id, Some(State::#state_name), &data).await.map_err(|e| TransitionError::ObserverError(e))?;
                                 Ok(#parent_name::<#next_state_name, T>::new(self.id, #next_state_name::new(data), self.observer))
@@ -187,9 +197,9 @@ pub fn statemachine(input: TokenStream) -> TokenStream {
                     Some(_) => quote! {
                         impl<T: Observer + Send> #parent_name<#state_name, T> {
                             pub async fn #event(mut self) -> Result<#parent_name<#next_state_name, T>, TransitionError<T::Error>> {
-                                self.observer.on_transition(&self.id, State::#state_name, State::#next_state_name, Option::<()>::None).await.map_err(|e| TransitionError::ObserverError(e))?;
+                                self.observer.on_transition(&self.id, State::#state_name, State::#next_state_name, Some(&self.data), Option::<()>::None).await.map_err(|e| TransitionError::ObserverError(e))?;
                                 #exit_call
-                                self.observer.#enter_fn_name(&self.id, Some(State::#state_name)).await.map_err(|e| TransitionError::ObserverError(e))?;
+                                self.observer.#enter_fn_name(&self.id, Some(State::#state_name), &self.data).await.map_err(|e| TransitionError::ObserverError(e))?;
                                 Ok(#parent_name::<#next_state_name, T>::new(self.id, #next_state_name::new(), self.data, self.observer))
                             }
                         }
@@ -197,7 +207,7 @@ pub fn statemachine(input: TokenStream) -> TokenStream {
                     None => quote! {
                         impl<T: Observer + Send> #parent_name<#state_name, T> {
                             pub async fn #event(mut self) -> Result<#parent_name<#next_state_name, T>, TransitionError<T::Error>> {
-                                self.observer.on_transition(&self.id, State::#state_name, State::#next_state_name, Option::<()>::None).await.map_err(|e| TransitionError::ObserverError(e))?;
+                                self.observer.on_transition(&self.id, State::#state_name, State::#next_state_name, Option::<()>::None, Option::<()>::None).await.map_err(|e| TransitionError::ObserverError(e))?;
                                 #exit_call
                                 self.observer.#enter_fn_name(&self.id, Some(State::#state_name)).await.map_err(|e| TransitionError::ObserverError(e))?;
                                 Ok(#parent_name::<#next_state_name, T>::new(self.id, #next_state_name::new(), self.observer))
@@ -219,10 +229,15 @@ pub fn statemachine(input: TokenStream) -> TokenStream {
         let state_name = &x.name;
         let enter_fn_name = format_ident!("{}_{}", "on_enter", state_name.to_string().to_case(Case::Snake));
 
-        let id_fn = quote! {
+        let common_methods = quote! {
             pub fn id(&self) -> &str {
                 &self.id
             }
+
+            pub fn change_observer(mut self, observer: T) -> Self {
+                self.observer = observer;
+                self
+            }
         };
 
         match shared_data_type {
@@ -238,7 +253,7 @@ pub fn statemachine(input: TokenStream) -> TokenStream {
                             }
                         }
 
-                        #id_fn
+                        #common_methods
 
                         pub fn data(&self) -> &#sdt {
                             &self.data
@@ -255,7 +270,7 @@ pub fn statemachine(input: TokenStream) -> TokenStream {
                             impl<T: Observer + Send> #parent_name<#state_name, T> {
                                 pub async fn init(id: Option<String>, data: #sdt, state_data: #dt, mut observer: T) -> Result<Self, InitError<T::Error>> {
                                     let id = observer.on_init(id, State::#state_name, Some(&data), Some(&state_data)).await.map_err(|e| InitError::ObserverError(e))?.ok_or(InitError::EmptyId)?;
-                                    observer.#enter_fn_name(&id, None, &state_data).await.map_err(|e| InitError::ObserverError(e))?;
+                                    observer.#enter_fn_name(&id, None, &data, &state_data).await.map_err(|e| InitError::ObserverError(e))?;
                                     Ok(Self::new(id, #state_name::new(state_data), data, observer))
                                 }
                             }
@@ -266,7 +281,7 @@ pub fn statemachine(input: TokenStream) -> TokenStream {
                             impl<T: Observer + Send> #parent_name<#state_name, T> {
                                 pub async fn init(id: Option<String>, data: #sdt, mut observer: T) -> Result<Self, InitError<T::Error>> {
                                     let id = observer.on_init(id, State::#state_name, Some(&data), Option::<()>::None).await.map_err(|e| InitError::ObserverError(e))?.ok_or(InitError::EmptyId)?;
-                                    observer.#enter_fn_name(&id, None).await.map_err(|e| InitError::ObserverError(e))?;
+                                    observer.#enter_fn_name(&id, None, &data).await.map_err(|e| InitError::ObserverError(e))?;
                                     Ok(Self::new(id, #state_name::new(), data, observer))
                                 }
                             }
@@ -285,7 +300,7 @@ pub fn statemachine(input: TokenStream) -> TokenStream {
                             }
                         }
 
-                        #id_fn
+                        #common_methods
                     }
                 };
 
@@ -342,31 +357,31 @@ pub fn statemachine(input: TokenStream) -> TokenStream {
         let state_name = &x.name;
         let expected_state_dt = state_data_types.get(state_name);
 
-        let fn_name = format_ident!("{}_{}", "restore", state_name.to_string().to_lowercase());
+        let fn_name = format_ident!("{}_{}", "restore", state_name.to_string().to_case(Case::Snake));
 
         match shared_data_type {
             Some(shared_dt) => {
                 match expected_state_dt {
                     Some(state_dt) => quote! {
-                        async fn #fn_name<'a, 'b, T: Observer + Send>(id: String, shared_d_enc: Option<Encoded<'a>>, state_d_enc: Option<Encoded<'b>>, observer: T) -> Result<#wrapped_type<T>, RestoreError> {
+                        async fn #fn_name<T: Observer + Send>(id: String, shared_d_enc: Option<Encoded>, state_d_enc: Option<Encoded>, observer: T) -> Result<#wrapped_type<T>, RestoreError> {
                             let shared_d_enc_some = shared_d_enc.ok_or(RestoreError::EmptyData)?;
                             let shared_d: #shared_dt = match shared_d_enc_some {
-                                Encoded::Json(data) => serde_json::from_str(&data).ok().ok_or(RestoreError::InvalidData)?
+                                Encoded::Json(data) => serde_json::from_value(data).ok().ok_or(RestoreError::InvalidData)?
                             };
 
                             let state_d_enc_some = state_d_enc.ok_or(RestoreError::EmptyData)?;
                             let state_d: #state_dt = match state_d_enc_some {
-                                Encoded::Json(data) => serde_json::from_str(&data).ok().ok_or(RestoreError::InvalidData)?
+                                Encoded::Json(data) => serde_json::from_value(data).ok().ok_or(RestoreError::InvalidData)?
                             };
 
                             Ok(#wrapped_type::#state_name(#parent_name::<#state_name, T>::new(id, #state_name::new(state_d), shared_d, observer)))
                         }
                     },
                     None => quote! {
-                        async fn #fn_name<'a, 'b, T: Observer + Send>(id: String, shared_d_enc: Option<Encoded<'a>>, state_d_enc: Option<Encoded<'b>>, observer: T) -> Result<#wrapped_type<T>, RestoreError> {
+                        async fn #fn_name<T: Observer + Send>(id: String, shared_d_enc: Option<Encoded>, state_d_enc: Option<Encoded>, observer: T) -> Result<#wrapped_type<T>, RestoreError> {
                             let shared_d_enc_some = shared_d_enc.ok_or(RestoreError::EmptyData)?;
                             let shared_d: #shared_dt = match shared_d_enc_some {
-                                Encoded::Json(data) => serde_json::from_str(&data).ok().ok_or(RestoreError::InvalidData)?
+                                Encoded::Json(data) => serde_json::from_value(data).ok().ok_or(RestoreError::InvalidData)?
                             };
 
                             if state_d_enc.is_some() {
@@ -381,21 +396,21 @@ pub fn statemachine(input: TokenStream) -> TokenStream {
             None => {
                 match expected_state_dt {
                     Some(state_dt) => quote! {
-                        async fn #fn_name<'a, 'b, T: Observer + Send>(id: String, shared_d_enc: Option<Encoded<'a>>, state_d_enc: Option<Encoded<'b>>, observer: T) -> Result<#wrapped_type<T>, RestoreError> {
+                        async fn #fn_name<T: Observer + Send>(id: String, shared_d_enc: Option<Encoded>, state_d_enc: Option<Encoded>, observer: T) -> Result<#wrapped_type<T>, RestoreError> {
                             if shared_d_enc.is_some() {
                                 return Err(RestoreError::UnexpectedData)
                             };
 
                             let state_d_enc_some = state_d_enc.ok_or(RestoreError::EmptyData)?;
                             let state_d: #state_dt = match state_d_enc_some {
-                                Encoded::Json(data) => serde_json::from_str(&data).ok().ok_or(RestoreError::InvalidData)?
+                                Encoded::Json(data) => serde_json::from_value(data).ok().ok_or(RestoreError::InvalidData)?
                             };
 
                             Ok(#wrapped_type::#state_name(#parent_name::<#state_name, T>::new(id, #state_name::new(state_d), observer)))
                         }
                     },
                     None => quote! {
-                        async fn #fn_name<'a, 'b, T: Observer + Send>(id: String, shared_d_enc: Option<Encoded<'a>>, state_d_enc: Option<Encoded<'b>>, observer: T) -> Result<#wrapped_type<T>, RestoreError> {
+                        async fn #fn_name<T: Observer + Send>(id: String, shared_d_enc: Option<Encoded>, state_d_enc: Option<Encoded>, observer: T) -> Result<#wrapped_type<T>, RestoreError> {
                             if shared_d_enc.is_some() {
                                 return Err(RestoreError::UnexpectedData)
                             };
@@ -414,7 +429,7 @@ pub fn statemachine(input: TokenStream) -> TokenStream {
 
     let restore_arms = m.states.iter().map(|x| {
         let state_name = &x.name;
-        let fn_name = format_ident!("{}_{}", "restore", state_name.to_string().to_lowercase());
+        let fn_name = format_ident!("{}_{}", "restore", state_name.to_string().to_case(Case::Snake));
         quote!(stringify!(#state_name) => #fn_name(id, data, state_data, observer).await)
     });
 
@@ -424,21 +439,41 @@ pub fn statemachine(input: TokenStream) -> TokenStream {
         let exit_fn_name = format_ident!("{}_{}", "on_exit", state_name.to_string().to_case(Case::Snake));
 
         let maybe_data_type = state_data_types.get(state_name);
-        match maybe_data_type {
-            Some(data_type) => quote! {
-                async fn #enter_fn_name(&mut self, id: &str, from: Option<State>, data: &#data_type) -> Result<(), Self::Error> {
-                    Ok(())
-                }
-                async fn #exit_fn_name(&mut self, id: &str, to: State, data: &#data_type) -> Result<(), Self::Error> {
-                    Ok(())
+        match shared_data_type {
+            Some(sdt) => match maybe_data_type {
+                Some(data_type) => quote! {
+                    async fn #enter_fn_name(&mut self, id: &str, from: Option<State>, data: &#sdt, state_data: &#data_type) -> Result<(), Self::Error> {
+                        Ok(())
+                    }
+                    async fn #exit_fn_name(&mut self, id: &str, to: State, data: &#sdt, state_data: &#data_type) -> Result<(), Self::Error> {
+                        Ok(())
+                    }
+                },
+                None => quote! {
+                    async fn #enter_fn_name(&mut self, id: &str, from: Option<State>, data: &#sdt) -> Result<(), Self::Error> {
+                        Ok(())
+                    }
+                    async fn #exit_fn_name(&mut self, id: &str, to: State, data: &#sdt) -> Result<(), Self::Error> {
+                        Ok(())
+                    }
                 }
             },
-            None => quote! {
-                async fn #enter_fn_name(&mut self, id: &str, from: Option<State>) -> Result<(), Self::Error> {
-                    Ok(())
-                }
-                async fn #exit_fn_name(&mut self, id: &str, to: State) -> Result<(), Self::Error> {
-                    Ok(())
+            None => match maybe_data_type {
+                Some(data_type) => quote! {
+                    async fn #enter_fn_name(&mut self, id: &str, from: Option<State>, state_data: &#data_type) -> Result<(), Self::Error> {
+                        Ok(())
+                    }
+                    async fn #exit_fn_name(&mut self, id: &str, to: State, state_data: &#data_type) -> Result<(), Self::Error> {
+                        Ok(())
+                    }
+                },
+                None => quote! {
+                    async fn #enter_fn_name(&mut self, id: &str, from: Option<State>) -> Result<(), Self::Error> {
+                        Ok(())
+                    }
+                    async fn #exit_fn_name(&mut self, id: &str, to: State) -> Result<(), Self::Error> {
+                        Ok(())
+                    }
                 }
             }
         }
@@ -464,6 +499,12 @@ pub fn statemachine(input: TokenStream) -> TokenStream {
             InvalidData,
             InvalidState
         }
+
+        #[derive(Debug)]
+        pub enum RetrieveError<T> {
+            RestoreError(RestoreError),
+            RetrieverError(T)
+        }
         
         pub enum State {
             #(#state_names),*
@@ -485,13 +526,20 @@ pub fn statemachine(input: TokenStream) -> TokenStream {
                 Ok(id)
             }
             
-            async fn on_transition<T: Serialize + Send>(&mut self, id: &str, from: State, to: State, data: Option<T>) -> Result<(), Self::Error> {
+            async fn on_transition<T: Serialize + Send, U: Serialize + Send>(&mut self, id: &str, from: State, to: State, data: Option<T>, state_data: Option<U>) -> Result<(), Self::Error> {
                 Ok(())
             }
 
             #(#listeners)*
         }
 
+        #[async_trait]
+        pub trait Retriever {
+            type RetrieverError;
+
+            async fn on_retrieve(&mut self, id: &str) -> Result<(String, Option<Encoded>, Option<Encoded>), Self::RetrieverError>;
+        }
+
         #parent_struct
         #(#state_structs)*
         #(#parent_state_impls)*
@@ -501,18 +549,25 @@ pub fn statemachine(input: TokenStream) -> TokenStream {
             #(#state_names(#parent_name<#state_names, T>)),*
         }
 
-        pub enum Encoded<'a> {
-            Json(&'a str)
+        pub enum Encoded {
+            Json(serde_json::Value)
         }
 
         #(#restore_fns)*
 
-        pub async fn restore<'a, 'b, T: Observer + Send>(id: String, state_str: &str, data: Option<Encoded<'a>>, state_data: Option<Encoded<'b>>, observer: T) -> Result<#wrapped_type<T>, RestoreError> {
+        pub async fn restore<T: Observer + Send>(id: String, state_string: String, data: Option<Encoded>, state_data: Option<Encoded>, observer: T) -> Result<#wrapped_type<T>, RestoreError> {
+            let state_str: &str = &state_string;
             match state_str {
                 #(#restore_arms,)*
                 _ => Err(RestoreError::InvalidState)
             }
         }
+
+        pub async fn retrieve<T: Observer + Retriever + Send>(id: String, mut obret: T) -> Result<#wrapped_type<T>, RetrieveError<T::RetrieverError>> {
+            let id_str: &str = &id;
+            let (state_string, maybe_data, maybe_state_data) = obret.on_retrieve(id_str).await.map_err(|e| RetrieveError::RetrieverError(e))?;
+            restore(id, state_string, maybe_data, maybe_state_data, obret).await.map_err(|e| RetrieveError::RestoreError(e))
+        }
     };
 
     out.into()
