@@ -1,8 +1,8 @@
 use async_trait::async_trait;
-use futures::executor::block_on;
-use serde::{ Serialize, Deserialize };
-use serde_json;
 use automato::statemachine;
+use futures::executor::block_on;
+use serde::{Deserialize, Serialize};
+use serde_json;
 
 #[derive(Serialize, Deserialize)]
 pub struct JobData {}
@@ -38,7 +38,13 @@ impl Observer for Log {
 
 #[test]
 fn init() {
-    let _job = block_on(Job::init(Some("foo".to_string()), JobData {}, QueuedData {}, Log {})).unwrap();
+    let _job = block_on(Job::init(
+        Some("foo".to_string()),
+        JobData {},
+        QueuedData {},
+        Log {},
+    ))
+    .unwrap();
 }
 
 #[test]
@@ -46,59 +52,88 @@ fn init_without_id() {
     let result = block_on(Job::init(None, JobData {}, QueuedData {}, Log {}));
     let err = result.err().unwrap();
     match err {
-        InitError::EmptyId => {},
-        _ => panic!("expected InitErr::EmptyId")
+        InitError::EmptyId => {}
+        _ => panic!("expected InitErr::EmptyId"),
     };
 }
 
 #[test]
 fn init_with_deferred_id() {
-    struct DeferredIdInitLog {
-    }
+    struct DeferredIdInitLog {}
 
     #[async_trait]
     impl Observer for DeferredIdInitLog {
         type Error = ();
 
-        async fn on_init<T:Serialize + Send, U:Serialize + Send>(&mut self, _id: Option<String>, _to: State, _data: Option<T> , _state_data: Option<U>) -> Result<Option<String>, Self::Error> {
+        async fn on_init<T: Serialize + Send, U: Serialize + Send>(
+            &mut self,
+            _id: Option<String>,
+            _to: State,
+            _data: Option<T>,
+            _state_data: Option<U>,
+        ) -> Result<Option<String>, Self::Error> {
             Ok(Some("foo".to_string()))
         }
     }
 
-    let _job = block_on(Job::init(None, JobData {}, QueuedData {}, DeferredIdInitLog {})).unwrap();
+    let _job = block_on(Job::init(
+        None,
+        JobData {},
+        QueuedData {},
+        DeferredIdInitLog {},
+    ))
+    .unwrap();
 }
 
 #[test]
 fn on_init() {
     struct InitLog {
-        initiated_to_state: Option<State>
+        initiated_to_state: Option<State>,
     }
-    
+
     #[async_trait]
     impl Observer for &mut InitLog {
         type Error = ();
 
-        async fn on_init<T:Serialize + Send, U:Serialize + Send>(&mut self, id: Option<String>, to: State, _data:Option<T> , _state_data:Option<U>) -> Result<Option<String>, Self::Error> {
+        async fn on_init<T: Serialize + Send, U: Serialize + Send>(
+            &mut self,
+            id: Option<String>,
+            to: State,
+            _data: Option<T>,
+            _state_data: Option<U>,
+        ) -> Result<Option<String>, Self::Error> {
             self.initiated_to_state = Some(to);
             Ok(id)
         }
     }
 
     let mut init_log = InitLog {
-        initiated_to_state: None
+        initiated_to_state: None,
     };
 
-    let _job = block_on(Job::init(Some("foo".to_string()), JobData {}, QueuedData {}, &mut init_log)).unwrap();
+    let _job = block_on(Job::init(
+        Some("foo".to_string()),
+        JobData {},
+        QueuedData {},
+        &mut init_log,
+    ))
+    .unwrap();
 
     match init_log.initiated_to_state {
         Some(state) => assert_eq!("Queued", state.to_string()),
-        None => panic!("expected some initiated_to_state value")
+        None => panic!("expected some initiated_to_state value"),
     };
 }
 
 #[test]
 fn read_id() {
-    let job = block_on(Job::init(Some("foo".to_string()), JobData {}, QueuedData {}, Log {})).unwrap();
+    let job = block_on(Job::init(
+        Some("foo".to_string()),
+        JobData {},
+        QueuedData {},
+        Log {},
+    ))
+    .unwrap();
     let id = job.id();
 
     assert_eq!(id, "foo");
@@ -106,14 +141,26 @@ fn read_id() {
 
 #[test]
 fn read_data() {
-    let job = block_on(Job::init(Some("foo".to_string()), JobData {}, QueuedData {}, Log {})).unwrap();
+    let job = block_on(Job::init(
+        Some("foo".to_string()),
+        JobData {},
+        QueuedData {},
+        Log {},
+    ))
+    .unwrap();
     let _job_data = job.data();
     let _job_state_data = job.state.data();
 }
 
 #[test]
 fn transition() {
-    let job = block_on(Job::init(Some("foo".to_string()), JobData {}, QueuedData {}, Log {})).unwrap();
+    let job = block_on(Job::init(
+        Some("foo".to_string()),
+        JobData {},
+        QueuedData {},
+        Log {},
+    ))
+    .unwrap();
     let _job = block_on(job.start(ProcessingData {})).unwrap();
 }
 
@@ -121,14 +168,21 @@ fn transition() {
 fn on_transition() {
     struct TransitionLog {
         from: Option<State>,
-        to: Option<State>
+        to: Option<State>,
     }
 
     #[async_trait]
     impl Observer for &mut TransitionLog {
         type Error = ();
 
-        async fn on_transition<T:Serialize + Send>(&mut self, _id: &str, from: State, to: State, _data: Option<T>) ->Result<(),Self::Error> {
+        async fn on_transition<T: Serialize + Send, U: Serialize + Send>(
+            &mut self,
+            _id: &str,
+            from: State,
+            to: State,
+            _data: Option<T>,
+            _state_data: Option<U>,
+        ) -> Result<(), Self::Error> {
             self.from = Some(from);
             self.to = Some(to);
             Ok(())
@@ -137,19 +191,25 @@ fn on_transition() {
 
     let mut transition_log = TransitionLog {
         from: None,
-        to: None
+        to: None,
     };
 
-    let job = block_on(Job::init(Some("foo".to_string()), JobData {}, QueuedData {}, &mut transition_log)).unwrap();
+    let job = block_on(Job::init(
+        Some("foo".to_string()),
+        JobData {},
+        QueuedData {},
+        &mut transition_log,
+    ))
+    .unwrap();
     let _job = block_on(job.start(ProcessingData {})).unwrap();
 
     match transition_log.from {
         Some(state) => assert_eq!("Queued", state.to_string()),
-        None => panic!("expected some from value")
+        None => panic!("expected some from value"),
     };
 
     match transition_log.to {
         Some(state) => assert_eq!("Processing", state.to_string()),
-        None => panic!("expected some to value")
+        None => panic!("expected some to value"),
     };
-}
\ No newline at end of file
+}
